@@ -1,5 +1,4 @@
-use std::sync::{RwLockReadGuard, RwLockWriteGuard};
-
+use std::{ops::{Deref, DerefMut}, sync::{RwLockReadGuard, RwLockWriteGuard}};
 use crate::{entity::EntityId, sparse_set::SparseSet};
 
 /// The Component trait  
@@ -63,19 +62,19 @@ impl dyn 'static + ComponentStorage {
 }
 
 
-pub struct ComponentRead<'a,T>{
+pub struct StorageRead<'a,T>{
     _lock : RwLockReadGuard<'a,Box<dyn ComponentStorage>>,
     ptr : *const SparseSet<EntityId,T>
 }
 
-impl<'a,T : Component> ComponentRead<'a,T> {
+impl<'a,T : Component> StorageRead<'a,T> {
     pub(in crate) fn from_lock(lock : RwLockReadGuard<'a,Box<dyn ComponentStorage>>) -> Self {
         // Safety:
         // 1.box has type SparseSet<EntityId,T>
         let ptr = unsafe {
             lock.downcast_ref::<SparseSet<EntityId,T>>()
         } as *const _;
-        ComponentRead {
+        StorageRead {
             _lock : lock,
             ptr,
         }
@@ -116,19 +115,19 @@ impl<'a,T : Component> ComponentRead<'a,T> {
 
 
 
-pub struct ComponentWrite<'a,T>{
+pub struct StorageWrite<'a,T>{
     _lock : RwLockWriteGuard<'a,Box<dyn ComponentStorage>>,
     ptr : *mut SparseSet<EntityId,T>
 }
 
-impl<'a,T : Component> ComponentWrite<'a,T> {
+impl<'a,T : Component> StorageWrite<'a,T> {
     pub(in crate) fn from_lock(mut lock : RwLockWriteGuard<'a,Box<dyn ComponentStorage>>) -> Self {
         // Safety:
         // 1.box has type SparseSet<EntityId,T>
         let ptr = unsafe {
             lock.downcast_mut::<SparseSet<EntityId,T>>()
         } as *mut _;
-        ComponentWrite{
+        StorageWrite{
             _lock : lock,
             ptr,
         }
@@ -180,3 +179,73 @@ impl<'a,T : Component> ComponentWrite<'a,T> {
     }
 }
 
+
+
+
+pub struct ComponentRead<'a,T> {
+    id : EntityId,
+    storage : StorageRead<'a,T>
+}
+
+impl<'a,T : Component> ComponentRead<'a,T> {
+    /// Safety:
+    /// Safe only id is valid
+    pub(in crate) unsafe fn new(id : EntityId,storage : StorageRead<'a,T>) -> Self {
+        ComponentRead{
+            id,
+            storage,
+        }
+    }
+}
+
+impl<'a,T : Component> Deref for ComponentRead<'a,T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        // Safety:
+        // new() method ensures this action is safe
+        unsafe {
+            self.storage.get_unchecked(self.id)
+        }
+    }
+}
+
+
+
+
+pub struct ComponentWrite<'a,T> {
+    id : EntityId,
+    storage : StorageWrite<'a,T>
+}
+
+impl<'a,T : Component> ComponentWrite<'a,T> {
+    /// Safety:
+    /// Safe only id is valid
+    pub(in crate) unsafe fn new(id : EntityId,storage : StorageWrite<'a,T>) -> Self {
+        ComponentWrite{
+            id,
+            storage,
+        }
+    }
+}
+
+impl<'a,T : Component> Deref for ComponentWrite<'a,T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        // Safety:
+        // new() method ensures this action is safe
+        unsafe {
+            self.storage.get_unchecked(self.id)
+        }
+    }
+}
+impl<'a,T : Component> DerefMut for ComponentWrite<'a,T> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        // Safety:
+        // new() method ensures this action is safe
+        unsafe {
+            self.storage.get_unchecked_mut(self.id)
+        }
+    }
+}
