@@ -9,30 +9,36 @@
 //! next ```world.create_entity()``` is called, it will allocate this ID to fill 
 //! the pit.Thanks to sparse set, it's still fast to 
 //! iterate all components no matter how random of ID
-use std::num::NonZeroUsize;
-use crate::component::Component;
-use crate::world::World;
+use std::{num::NonZeroUsize, sync::RwLockReadGuard};
+use crate::{component::{Component, ComponentRead, ComponentWrite}, world::World};
 
 /// The type of ID of entity which starts from 1 and can be recycled automatically
 pub type EntityId = NonZeroUsize;
 
 /// A useful struct for building a entity
-#[derive(Debug)]
-pub struct EntityBuilder<'a>{
-    world : &'a mut World,
+// #[derive(Debug)]
+pub struct Entity<'a>{
+    world : &'a World,
     id : EntityId,
+    // To avoid remove this ID from world
+    // The ID must be valid during Entity is alive
+    #[allow(unused)]
+    borrow_entity_manager : RwLockReadGuard<'a,EntityManager>
 }
 
-impl<'a> EntityBuilder<'a>{
-    pub(in crate) fn new(world : &'a mut World,entity_id : EntityId) -> Self{
-        EntityBuilder{
+impl<'a> Entity<'a>{
+    pub(in crate) fn new(world : &'a World,
+                         borrow_entity_manager : RwLockReadGuard<'a,EntityManager>,
+                         entity_id : EntityId) -> Self{
+        Entity{
             world,
             id: entity_id,
+            borrow_entity_manager,
         }
     }
 
-    /// Consume EntityRef and return a valid EntityId
-    pub fn into_id(self) -> EntityId{
+    /// Get entity id
+    pub fn id(&self) -> EntityId{
         self.id
     }
 
@@ -43,11 +49,19 @@ impl<'a> EntityBuilder<'a>{
     }
 
     /// Detach a component from entity
-    pub fn detach<T : Component>(self) -> Self{
-        self.world.detach_component::<T>(self.id);//ignore the error
-        self
+    pub fn detach<T : Component>(&self) -> Option<T> {
+        self.world.detach_component(self.id)
     }
 
+    /// Read component of this entity
+    pub fn component_read<T : Component>(&self) -> Option<ComponentRead<'_,T>> {
+        self.world.entity_component_read(self.id)
+    }
+
+    /// Write component of this entity
+    pub fn component_write<T : Component>(&self) -> Option<ComponentWrite<'_,T>> {
+        self.world.entity_component_write(self.id)
+    }
 }
 
 #[derive(Debug,Copy,Clone)]
@@ -123,9 +137,9 @@ impl EntityManager {
         }
     }
 
-    pub(in crate) fn entities(&self) -> &[EntityId] {
-        self.entities.as_slice()
-    }
+    // pub(in crate) fn entities(&self) -> &[EntityId] {
+        // self.entities.as_slice()
+    // }
 
     #[allow(dead_code)]
     pub(in crate) fn len(&self) -> usize {
@@ -187,4 +201,5 @@ mod tests{
         println!("entities :{:?}",manager.entities.as_slice());
         println!();
     }
+    
 }
