@@ -8,7 +8,7 @@ pub struct IterRefRef<'a,A,B> {
     sparse_set_a: *const SparseSet<EntityId,A>,
     sparse_set_b: *const SparseSet<EntityId,B>,
     #[allow(unused)]
-    borrow_group: RwLockReadGuard<'a,Box<dyn Group>>,
+    borrow_group: RwLockReadGuard<'a,Group>,
     #[allow(unused)]
     borrow_a: RwLockReadGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
@@ -25,8 +25,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a A,&'a B> {
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let storage_a = world.storage_ref(type_id_a).unwrap();
-        let storage_b = world.storage_ref(type_id_b).unwrap();
+        let storage_a = world.raw_storage_read(type_id_a).unwrap();
+        let storage_b = world.raw_storage_read(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -37,13 +37,11 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a A,&'a B> {
         };
         let ptr_a = &*sparse_set_a;
         let ptr_b = &*sparse_set_b;
-        let group = non_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for NonOwning: Group is not in world");
-        let group = world.group(&group);
-        // Safety:
-        // group type is NonOwning<A,B>
-        let group_data = unsafe {
-            group.downcast_ref::<NonOwning<A,B>>()
+        assert!(world.has_group(non_owning::<A,B>()),"Queryable for NonOwning: Group is not in world");
+        let group = world.group(non_owning::<A,B>());
+        let group_data = match &*group {
+            Group::NonOwning(data) => data,
+            _ => unreachable!()
         };
         let group_data = &group_data.sparse_set;
         let ptr_group = &*group_data;
@@ -175,7 +173,7 @@ pub struct IterRefMut<'a,A,B> {
     sparse_set_a: *const SparseSet<EntityId,A>,
     sparse_set_b: *mut SparseSet<EntityId,B>,
     #[allow(unused)]
-    borrow_group: RwLockReadGuard<'a,Box<dyn Group>>,
+    borrow_group: RwLockReadGuard<'a,Group>,
     #[allow(unused)]
     borrow_a: RwLockReadGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
@@ -192,8 +190,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a A,&'a mut B
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let storage_a = world.storage_ref(type_id_a).unwrap();
-        let mut storage_b = world.storage_mut(type_id_b).unwrap();
+        let storage_a = world.raw_storage_read(type_id_a).unwrap();
+        let mut storage_b = world.raw_storage_write(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -204,13 +202,11 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a A,&'a mut B
         };
         let ptr_a = &*sparse_set_a;
         let ptr_b = &mut *sparse_set_b;
-        let group = non_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for NonOwning: Group is not in world");
-        let group = world.group(&group);
-        // Safety:
-        // group type is NonOwning<A,B>
-        let group_data = unsafe {
-            group.downcast_ref::<NonOwning<A,B>>()
+        assert!(world.has_group(non_owning::<A,B>()),"Queryable for NonOwning: Group is not in world");
+        let group = world.group(non_owning::<A,B>());
+        let group_data = match &*group {
+            Group::NonOwning(data) => data,
+            _ => unreachable!()
         };
         let group_data = &group_data.sparse_set;
         let ptr_group = &*group_data;
@@ -342,7 +338,7 @@ pub struct IterMutRef<'a,A,B> {
     sparse_set_a: *mut SparseSet<EntityId,A>,
     sparse_set_b: *const SparseSet<EntityId,B>,
     #[allow(unused)]
-    borrow_group: RwLockReadGuard<'a,Box<dyn Group>>,
+    borrow_group: RwLockReadGuard<'a,Group>,
     #[allow(unused)]
     borrow_a: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
@@ -359,8 +355,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a mut A,&'a B
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let mut storage_a = world.storage_mut(type_id_a).unwrap();
-        let storage_b = world.storage_ref(type_id_b).unwrap();
+        let mut storage_a = world.raw_storage_write(type_id_a).unwrap();
+        let storage_b = world.raw_storage_read(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -371,13 +367,11 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a mut A,&'a B
         };
         let ptr_a = &mut *sparse_set_a;
         let ptr_b = &*sparse_set_b;
-        let group = non_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for NonOwning: Group is not in world");
-        let group = world.group(&group);
-        // Safety:
-        // group type is NonOwning<A,B>
-        let group_data = unsafe {
-            group.downcast_ref::<NonOwning<A,B>>()
+        assert!(world.has_group(non_owning::<A,B>()),"Queryable for NonOwning: Group is not in world");
+        let group = world.group(non_owning::<A,B>());
+        let group_data = match &*group {
+            Group::NonOwning(data) => data,
+            _ => unreachable!()
         };
         let group_data = &group_data.sparse_set;
         let ptr_group = &*group_data;
@@ -510,7 +504,7 @@ pub struct IterMutMut<'a,A,B> {
     sparse_set_a: *mut SparseSet<EntityId,A>,
     sparse_set_b: *mut SparseSet<EntityId,B>,
     #[allow(unused)]
-    borrow_group: RwLockReadGuard<'a,Box<dyn Group>>,
+    borrow_group: RwLockReadGuard<'a,Group>,
     #[allow(unused)]
     borrow_a: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
@@ -527,8 +521,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a mut A,&'a m
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let mut storage_a = world.storage_mut(type_id_a).unwrap();
-        let mut storage_b = world.storage_mut(type_id_b).unwrap();
+        let mut storage_a = world.raw_storage_write(type_id_a).unwrap();
+        let mut storage_b = world.raw_storage_write(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -539,13 +533,11 @@ impl<'a,A : Component,B : Component> Queryable<'a> for NonOwning<&'a mut A,&'a m
         };
         let ptr_a = &mut *sparse_set_a;
         let ptr_b = &mut *sparse_set_b;
-        let group = non_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for NonOwning: Group is not in world");
-        let group = world.group(&group);
-        // Safety:
-        // group type is NonOwning<A,B>
-        let group_data = unsafe {
-            group.downcast_ref::<NonOwning<A,B>>()
+        assert!(world.has_group(non_owning::<A,B>()),"Queryable for NonOwning: Group is not in world");
+        let group = world.group(non_owning::<A,B>());
+        let group_data = match &*group {
+            Group::NonOwning(data) => data,
+            _ => unreachable!()
         };
         let group_data = &group_data.sparse_set;
         let ptr_group = &*group_data;
