@@ -1,5 +1,5 @@
 use std::{any::TypeId, sync::{RwLockReadGuard, RwLockWriteGuard}};
-use crate::{component::{Component, ComponentStorage}, entity::EntityId, group::partial_owning, query::{QueryIterator, Queryable}, sparse_set::SparseSet, world::World};
+use crate::{component::{Component, ComponentStorage}, entity::EntityId, group::{Group, partial_owning}, query::{QueryIterator, Queryable}, sparse_set::SparseSet, world::World};
 use super::PartialOwning;
 
 pub struct IterRefRef<'a,A,B> {
@@ -10,7 +10,9 @@ pub struct IterRefRef<'a,A,B> {
     #[allow(unused)]
     borrow_a: RwLockReadGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
-    borrow_b: RwLockReadGuard<'a,Box<dyn ComponentStorage>>
+    borrow_b: RwLockReadGuard<'a,Box<dyn ComponentStorage>>,
+    #[allow(unused)]
+    borrow_group: RwLockReadGuard<'a,Group>
 }
 
 impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a A,&'a B> {
@@ -23,8 +25,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a A,&'a B
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let storage_a = world.storage_ref(type_id_a).unwrap();
-        let storage_b = world.storage_ref(type_id_b).unwrap();
+        let storage_a = world.raw_storage_read(type_id_a).unwrap();
+        let storage_b = world.raw_storage_read(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -35,16 +37,17 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a A,&'a B
         };
         let ptr_a = &*sparse_set_a;
         let ptr_b = &*sparse_set_b;
-        let group = partial_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for PartialOwning: Group is not in world");
-        let length = world.group(&group).len();
+        assert!(world.has_group(partial_owning::<A,B>()),"Queryable for PartialOwning: Group is not in world");
+        let group = world.group(partial_owning::<A,B>());
+        let length = group.len();
         Box::new(IterRefRef {
             index: 0,
             length,
             sparse_set_a: ptr_a,
             sparse_set_b: ptr_b,
             borrow_a: storage_a,
-            borrow_b: storage_b
+            borrow_b: storage_b,
+            borrow_group: group,
         })
     }
 }
@@ -148,7 +151,9 @@ pub struct IterRefMut<'a,A,B> {
     #[allow(unused)]
     borrow_a: RwLockReadGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
-    borrow_b: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>
+    borrow_b: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>,
+    #[allow(unused)]
+    borrow_group: RwLockReadGuard<'a,Group>
 }
 
 impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a A,&'a mut B> {
@@ -161,8 +166,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a A,&'a m
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let storage_a = world.storage_ref(type_id_a).unwrap();
-        let mut storage_b = world.storage_mut(type_id_b).unwrap();
+        let storage_a = world.raw_storage_read(type_id_a).unwrap();
+        let mut storage_b = world.raw_storage_write(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -173,16 +178,17 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a A,&'a m
         };
         let ptr_a = &*sparse_set_a;
         let ptr_b = &mut *sparse_set_b;
-        let group = partial_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for PartialOwning: Group is not in world");
-        let length = world.group(&group).len();
+        assert!(world.has_group(partial_owning::<A,B>()),"Queryable for PartialOwning: Group is not in world");
+        let group = world.group(partial_owning::<A,B>());
+        let length = group.len();
         Box::new(IterRefMut {
             index: 0,
             length,
             sparse_set_a: ptr_a,
             sparse_set_b: ptr_b,
             borrow_a: storage_a,
-            borrow_b: storage_b
+            borrow_b: storage_b,
+            borrow_group: group,
         })
     }
 }
@@ -288,7 +294,9 @@ pub struct IterMutRef<'a,A,B> {
     #[allow(unused)]
     borrow_a: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
-    borrow_b: RwLockReadGuard<'a,Box<dyn ComponentStorage>>
+    borrow_b: RwLockReadGuard<'a,Box<dyn ComponentStorage>>,
+    #[allow(unused)]
+    borrow_group: RwLockReadGuard<'a,Group>
 }
 
 impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a mut A,&'a B> {
@@ -301,8 +309,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a mut A,&
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let mut storage_a = world.storage_mut(type_id_a).unwrap();
-        let storage_b = world.storage_ref(type_id_b).unwrap();
+        let mut storage_a = world.raw_storage_write(type_id_a).unwrap();
+        let storage_b = world.raw_storage_read(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -313,16 +321,17 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a mut A,&
         };
         let ptr_a = &mut *sparse_set_a;
         let ptr_b = &*sparse_set_b;
-        let group = partial_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for PartialOwning: Group is not in world");
-        let length = world.group(&group).len();
+        assert!(world.has_group(partial_owning::<A,B>()),"Queryable for PartialOwning: Group is not in world");
+        let group = world.group(partial_owning::<A,B>());
+        let length = group.len();
         Box::new(IterMutRef {
             index: 0,
             length,
             sparse_set_a: ptr_a,
             sparse_set_b: ptr_b,
             borrow_a: storage_a,
-            borrow_b: storage_b
+            borrow_b: storage_b,
+            borrow_group: group,
         })
     }
 }
@@ -426,7 +435,9 @@ pub struct IterMutMut<'a,A,B> {
     #[allow(unused)]
     borrow_a: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>,
     #[allow(unused)]
-    borrow_b: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>
+    borrow_b: RwLockWriteGuard<'a,Box<dyn ComponentStorage>>,
+    #[allow(unused)]
+    borrow_group: RwLockReadGuard<'a,Group>
 }
 
 impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a mut A,&'a mut B> {
@@ -439,8 +450,8 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a mut A,&
         let type_id_b = TypeId::of::<B>();
         // Unwrap here
         // assert before ensures this
-        let mut storage_a = world.storage_mut(type_id_a).unwrap();
-        let mut storage_b = world.storage_mut(type_id_b).unwrap();
+        let mut storage_a = world.raw_storage_write(type_id_a).unwrap();
+        let mut storage_b = world.raw_storage_write(type_id_b).unwrap();
         // Safety:
         // storage is SparseSet<EntityId,...>
         let sparse_set_a = unsafe {
@@ -451,16 +462,17 @@ impl<'a,A : Component,B : Component> Queryable<'a> for PartialOwning<&'a mut A,&
         };
         let ptr_a = &mut *sparse_set_a;
         let ptr_b = &mut *sparse_set_b;
-        let group = partial_owning::<A,B>();
-        assert!(world.has_group(&group),"Queryable for PartialOwning: Group is not in world");
-        let length = world.group(&group).len();
+        assert!(world.has_group(partial_owning::<A,B>()),"Queryable for PartialOwning: Group is not in world");
+        let group = world.group(partial_owning::<A,B>());
+        let length = group.len();
         Box::new(IterMutMut {
             index: 0,
             length,
             sparse_set_a: ptr_a,
             sparse_set_b: ptr_b,
             borrow_a: storage_a,
-            borrow_b: storage_b
+            borrow_b: storage_b,
+            borrow_group: group,
         })
     }
 }
